/*global exports: false, require: false, console: false */
var pageMod = require("page-mod");
var data = require("self").data;
var tabs = require("tabs");
var self = require("self");
var Request = require("request").Request;
var passUtils = require("passwords");
var prompts = require("prompts");
var xmlrpc = require("xmlrpc");

var ourRealm = "massCloseDeduplicate";

/*
var xmlrpc = require("xmlrpc");
var myConfig = require("simple-storage").storage;
var timer = require("timer");
var urlMod = require("url");
*/

var howManyAtOnce = 10,
reqCounter = 0,
XMLRPCURLs = JSON.parse(self.data.load("XMLRPCdata.json"));

var Message = exports.Message = function Message(cmd, data) {
	this.cmd = cmd;
	this.data = data;
};

/**
 * Callback function for the XMLRPC request
 *
 * @param ret object with xmlhttprequest response
 *                with attributes:
 *                + status -- int return code
 *                + statusText
 *                + responseHeaders
 *                + responseText
 */
function callBack() {
    console.log("reqCounter = " + reqCounter);
    reqCounter -= 1;
    if (reqCounter <= 0) {
        processReqQueue();
    }
}

/**
 * The worker function -- call XMLRPC to fix MIME type of the
 * particular attachment
 *
 * @param id integer with the attachment id to be fixed
 * @param type string with the new MIME type, e.g. "text/plain"

    "closeBug($params);"
        Close a current Bugzilla bug report with a specific resolution.
        This will eventually be done in Bugzilla/Bug.pm instead and is
        meant to only be a quick fix. Please use RedHat.changesStatus to
        changed to an opened state.  This method will change the bug
        report's status to CLOSED.

        Params:
            $params => {
                id => '<bug_id>',             # ID of bug report to close
                resolution => '<resolution>', # Valid Bugzilla resolution
                                                to transition the report into.
                                                DUPLICATE requires 'dup_id'
                                                to be passed in.
                dup_id => '<bug_id>',         # Bugzilla report ID that
                                                this bug is being closed as
                                                duplicate of. Requires
                                                'resolution' to be DUPLICATE.
                fixed_in => '<fixed_in>',     # OPTIONAL String
                                                representing version of
                                                product/component
                                                that bug is fixed in.
                comment => '<comment>',       # OPTIONAL Text string
                                                containing comment to add.
                isprivate => 1 or 0,          # OPTIONAL Whether the
                                                comment will be private to the
                                                'private_comment' Bugzilla
                                                group.
                                                Default: false
                nomail => 1 or 0,             # OPTIONAL Flag that is
                                                either 1 or 0 if you want
                                                email to be sent or not for
                                                this change
            }
 */
function deduplicate(id, dID, login, password, host) {
    var XMLRPCURLs = JSON.parse(self.data.load("XMLRPCdata.json")),
    msg = new xmlrpc.XMLRPCMessage("Bug.update"); // upstream call!!!
    msg.addParameter({
        'ids' : id,
        'nomail': 1,
        'Bugzilla_login': login,
        'Bugzilla_password': password,
        updates: {
            'status': 'CLOSED',
            'resolution': "DUPLICATE",
            'dupe_id': dID
        }
    });
    Request({
        url: XMLRPCURLs[host].url, // FIXME
        onComplete: function(response) {
            if (response.status == 200) {
                callBack();
            } else {
                throw new Error("HTTP Request failed!");
            }
        },
        content: msg.xml(),
        contentType: "text/xml"
    }).post();
    reqCounter++;
}


function  processReqQueue(queue, dupID, login, pass, host, callback) {
    var howMany = 0;
    if (queue.length <= 0) {
         timer.setTimeout(callback, 1000);
        return ;
    } else {
        if (queue.length >= howManyAtOnce) {
            howMany = howManyAtOnce;
        } else {
            howMany = queue.length;
        }
        var part = queue.splice(0, howMany);
        part.forEach(function(id) {
            deduplicate(id, dupID, login, pass, host);
        }, this);

        timer.setTimeout(function () {
        	processReqQueue(queue, dupID, login, pass, host, callback);
        }, 1000);
    }
};

function getPassword(log, host, callback) {
	login = log;
	password = passUtils.getPassword(login,
		host, ourRealm);

	if (!password) {
		password = prompts.prompt("Enter your Bugzilla password");
		passUtils.setLogin(login, password, host, ourRealm);
		}
	callback(new Message("RetPassword", password));
}

function getDuplicatorID(hostname, closedBugID, callback) {
	var bugzilla = new XML(), dupeIDs = [], dupeID = null,
	statusElement = {}, resolutionElement = {};

	Request({
		url: "https://" + hostname + "/show_bug.cgi?ctype=xml&id=" + closedBugID,
		onComplete: function (event) {
			if (req.status === 200 && req.response.text) {
				var responseText = req.response.text.
					replace(/^<\?xml\s+version\s*=\s*(["'])[^\1]+\1[^?]*\?>/, "");
    			bugzilla = new XML(responseText);
				statusElement = bugzilla.bug.bug_status;
				console.log("statusElement = " + statusElement.text() + "/" + typeof(statusElement));
				resolutionElement = bugzilla.bug.resolution;
				console.log("resolutionElement = " + resolutionElement + "/" + typeof(resolutionElement));
				if ((statusElement && statusElement.text() != "CLOSED") &&
					(resolutionElement && resolutionElement.text() !== "DUPLICATE")) {
						callback(new Message("RetDuplicatorID",
							parseInt(bugzilla.bug.dup_id.text(), 10)));
				} else {
					callback(new Message("Error",
						"Either bug is not closed or it is not duplicate."));
				}
			}
		}
	});
}

var messageHandler = exports.messageHandler = function messageHandler(worker, msg) {
	switch (msg.cmd) {
		case "LogMessage":
			console.log(msg.data);
			break;
		case "GetDuplicatorID":
			getDuplicatorID(msg.data.host, msg.data.bugID, function (msgObj) {
				worker.postMessage(msgObj);
			});
			break;
		case "GetPassword":
			getPassword(msg.data.login, msg.data.hostname, function(pass) {
				worker.postMessage(pass);
			});
			break;
		case "DeDeduplicateQueue":
			processReqQueue(msg.data.bugs, msg.data.duplicator,
				msg.data.login, msg.data.password, msg.data.hostname, function() {
					worker.postMessage(new Message("ReloadThePage", null));
			});
		default:
			console.error(msg.toSource());
	}
};

pageMod.PageMod({
  include: [
  	"https://bugzilla.redhat.com/buglist.cgi?*"],
  contentScriptWhen: 'ready',
  contentScriptFile: [
  	self.data.url("pageWorker.js")
  ],
  onAttach: function onAttach(worker, mod) {
    worker.on('message', function (msg) {
    	messageHandler(worker, msg);
    });
  }
});
